@@ -8,13 +8,17 @@ from sklearn.model_selection import cross_val_predict, KFold, train_test_split
 from xgboost import XGBRegressor
 
 from causalml.inference.meta.base import BaseLearner
-from causalml.inference.meta.utils import (check_treatment_vector,
-    get_xgboost_objective_metric, convert_pd_to_np, get_weighted_variance)
+from causalml.inference.meta.utils import (
+    check_treatment_vector,
+    get_xgboost_objective_metric,
+    convert_pd_to_np,
+    get_weighted_variance,
+)
 from causalml.inference.meta.explainer import Explainer
 from causalml.propensity import compute_propensity_score, ElasticNetPropensityModel
 
 
-logger = logging.getLogger('causalml')
+logger = logging.getLogger("causalml")
 
 
 class BaseRLearner(BaseLearner):
@@ -25,15 +29,17 @@ class BaseRLearner(BaseLearner):
     Details of R-learner are available at Nie and Wager (2019) (https://arxiv.org/abs/1712.04912).
     """
 
-    def __init__(self,
-                 learner=None,
-                 outcome_learner=None,
-                 effect_learner=None,
-                 propensity_learner=ElasticNetPropensityModel(),
-                 ate_alpha=.05,
-                 control_name=0,
-                 n_fold=5,
-                 random_state=None):
+    def __init__(
+        self,
+        learner=None,
+        outcome_learner=None,
+        effect_learner=None,
+        propensity_learner=ElasticNetPropensityModel(),
+        ate_alpha=0.05,
+        control_name=0,
+        n_fold=5,
+        random_state=None,
+    ):
         """Initialize an R-learner.
 
         Args:
@@ -48,11 +54,17 @@ class BaseRLearner(BaseLearner):
             n_fold (int, optional): the number of cross validation folds for outcome_learner
             random_state (int or RandomState, optional): a seed (int) or random number generator (RandomState)
         """
-        assert (learner is not None) or ((outcome_learner is not None) and (effect_learner is not None))
+        assert (learner is not None) or (
+            (outcome_learner is not None) and (effect_learner is not None)
+        )
         assert propensity_learner is not None
 
-        self.model_mu = outcome_learner if outcome_learner is not None else deepcopy(learner)
-        self.model_tau = effect_learner if effect_learner is not None else deepcopy(learner)
+        self.model_mu = (
+            outcome_learner if outcome_learner is not None else deepcopy(learner)
+        )
+        self.model_tau = (
+            effect_learner if effect_learner is not None else deepcopy(learner)
+        )
         self.model_p = propensity_learner
 
         self.ate_alpha = ate_alpha
@@ -65,10 +77,12 @@ class BaseRLearner(BaseLearner):
         self.propensity_model = None
 
     def __repr__(self):
-        return (f'{self.__class__.__name__}\n'
-                f'\toutcome_learner={self.model_mu.__repr__()}\n'
-                f'\teffect_learner={self.model_tau.__repr__()}\n'
-                f'\tpropensity_learner={self.model_p.__repr__()}')
+        return (
+            f"{self.__class__.__name__}\n"
+            f"\toutcome_learner={self.model_mu.__repr__()}\n"
+            f"\teffect_learner={self.model_tau.__repr__()}\n"
+            f"\tpropensity_learner={self.model_p.__repr__()}"
+        )
 
     def fit(self, X, treatment, y, p=None, sample_weight=None, verbose=True):
         """Fit the treatment effect and outcome models of the R learner.
@@ -87,7 +101,9 @@ class BaseRLearner(BaseLearner):
         X, treatment, y = convert_pd_to_np(X, treatment, y)
         check_treatment_vector(treatment, self.control_name)
         if sample_weight is not None:
-            assert len(sample_weight) == len(y), "Data length must be equal for sample_weight and the input data"
+            assert len(sample_weight) == len(
+                y
+            ), "Data length must be equal for sample_weight and the input data"
             sample_weight = convert_pd_to_np(sample_weight)
         self.t_groups = np.unique(treatment[treatment != self.control_name])
         self.t_groups.sort()
@@ -104,7 +120,7 @@ class BaseRLearner(BaseLearner):
         self.vars_t = {}
 
         if verbose:
-            logger.info('generating out-of-fold CV outcome estimates')
+            logger.info("generating out-of-fold CV outcome estimates")
         yhat = cross_val_predict(self.model_mu, X, y, cv=self.cv, n_jobs=-1)
 
         for group in self.t_groups:
@@ -131,9 +147,14 @@ class BaseRLearner(BaseLearner):
                 self.vars_t[group] = diff_t.var()
 
             if verbose:
-                logger.info('training the treatment effect model for {} with R-loss'.format(group))
-            self.models_tau[group].fit(X_filt, (y_filt - yhat_filt) / (w - p_filt),
-                                       sample_weight=weight)
+                logger.info(
+                    "training the treatment effect model for {} with R-loss".format(
+                        group
+                    )
+                )
+            self.models_tau[group].fit(
+                X_filt, (y_filt - yhat_filt) / (w - p_filt), sample_weight=weight
+            )
 
     def predict(self, X, p=None):
         """Predict treatment effects.
@@ -152,8 +173,18 @@ class BaseRLearner(BaseLearner):
 
         return te
 
-    def fit_predict(self, X, treatment, y, p=None, sample_weight=None, return_ci=False,
-                    n_bootstraps=1000, bootstrap_size=10000, verbose=True):
+    def fit_predict(
+        self,
+        X,
+        treatment,
+        y,
+        p=None,
+        sample_weight=None,
+        return_ci=False,
+        n_bootstraps=1000,
+        bootstrap_size=10000,
+        verbose=True,
+    ):
         """Fit the treatment effect and outcome models of the R learner and predict treatment effects.
 
         Args:
@@ -185,9 +216,11 @@ class BaseRLearner(BaseLearner):
             _classes_global = self._classes
             model_mu_global = deepcopy(self.model_mu)
             models_tau_global = deepcopy(self.models_tau)
-            te_bootstraps = np.zeros(shape=(X.shape[0], self.t_groups.shape[0], n_bootstraps))
+            te_bootstraps = np.zeros(
+                shape=(X.shape[0], self.t_groups.shape[0], n_bootstraps)
+            )
 
-            logger.info('Bootstrap Confidence Intervals')
+            logger.info("Bootstrap Confidence Intervals")
             for i in tqdm(range(n_bootstraps)):
                 if p is None:
                     p = self.propensity
@@ -197,7 +230,9 @@ class BaseRLearner(BaseLearner):
                 te_bootstraps[:, :, i] = te_b
 
             te_lower = np.percentile(te_bootstraps, (self.ate_alpha / 2) * 100, axis=2)
-            te_upper = np.percentile(te_bootstraps, (1 - self.ate_alpha / 2) * 100, axis=2)
+            te_upper = np.percentile(
+                te_bootstraps, (1 - self.ate_alpha / 2) * 100, axis=2
+            )
 
             # set member variables back to global (currently last bootstrapped outcome)
             self.t_groups = t_groups_global
@@ -207,8 +242,17 @@ class BaseRLearner(BaseLearner):
 
             return (te, te_lower, te_upper)
 
-    def estimate_ate(self, X, treatment, y, p=None, sample_weight=None, bootstrap_ci=False,
-                     n_bootstraps=1000, bootstrap_size=10000):
+    def estimate_ate(
+        self,
+        X,
+        treatment,
+        y,
+        p=None,
+        sample_weight=None,
+        bootstrap_ci=False,
+        n_bootstraps=1000,
+        bootstrap_size=10000,
+    ):
         """Estimate the Average Treatment Effect (ATE).
 
         Args:
@@ -238,10 +282,14 @@ class BaseRLearner(BaseLearner):
             prob_treatment = float(sum(w)) / X.shape[0]
             _ate = te[:, i].mean()
 
-            se = (np.sqrt((self.vars_t[group] / prob_treatment)
-                          + (self.vars_c[group] / (1 - prob_treatment))
-                          + te[:, i].var())
-                  / X.shape[0])
+            se = (
+                np.sqrt(
+                    (self.vars_t[group] / prob_treatment)
+                    + (self.vars_c[group] / (1 - prob_treatment))
+                    + te[:, i].var()
+                )
+                / X.shape[0]
+            )
 
             _ate_lb = _ate - se * norm.ppf(1 - self.ate_alpha / 2)
             _ate_ub = _ate + se * norm.ppf(1 - self.ate_alpha / 2)
@@ -258,7 +306,7 @@ class BaseRLearner(BaseLearner):
             model_mu_global = deepcopy(self.model_mu)
             models_tau_global = deepcopy(self.models_tau)
 
-            logger.info('Bootstrap Confidence Intervals for ATE')
+            logger.info("Bootstrap Confidence Intervals for ATE")
             ate_bootstraps = np.zeros(shape=(self.t_groups.shape[0], n_bootstraps))
 
             for n in tqdm(range(n_bootstraps)):
@@ -269,8 +317,12 @@ class BaseRLearner(BaseLearner):
                 cate_b = self.bootstrap(X, treatment, y, p, size=bootstrap_size)
                 ate_bootstraps[:, n] = cate_b.mean()
 
-            ate_lower = np.percentile(ate_bootstraps, (self.ate_alpha / 2) * 100, axis=1)
-            ate_upper = np.percentile(ate_bootstraps, (1 - self.ate_alpha / 2) * 100, axis=1)
+            ate_lower = np.percentile(
+                ate_bootstraps, (self.ate_alpha / 2) * 100, axis=1
+            )
+            ate_upper = np.percentile(
+                ate_bootstraps, (1 - self.ate_alpha / 2) * 100, axis=1
+            )
 
             # set member variables back to global (currently last bootstrapped outcome)
             self.t_groups = t_groups_global
@@ -285,15 +337,17 @@ class BaseRRegressor(BaseRLearner):
     A parent class for R-learner regressor classes.
     """
 
-    def __init__(self,
-                 learner=None,
-                 outcome_learner=None,
-                 effect_learner=None,
-                 propensity_learner=ElasticNetPropensityModel(),
-                 ate_alpha=.05,
-                 control_name=0,
-                 n_fold=5,
-                 random_state=None):
+    def __init__(
+        self,
+        learner=None,
+        outcome_learner=None,
+        effect_learner=None,
+        propensity_learner=ElasticNetPropensityModel(),
+        ate_alpha=0.05,
+        control_name=0,
+        n_fold=5,
+        random_state=None,
+    ):
         """Initialize an R-learner regressor.
 
         Args:
@@ -316,7 +370,8 @@ class BaseRRegressor(BaseRLearner):
             ate_alpha=ate_alpha,
             control_name=control_name,
             n_fold=n_fold,
-            random_state=random_state)
+            random_state=random_state,
+        )
 
 
 class BaseRClassifier(BaseRLearner):
@@ -324,14 +379,16 @@ class BaseRClassifier(BaseRLearner):
     A parent class for R-learner classifier classes.
     """
 
-    def __init__(self,
-                 outcome_learner=None,
-                 effect_learner=None,
-                 propensity_learner=ElasticNetPropensityModel(),
-                 ate_alpha=.05,
-                 control_name=0,
-                 n_fold=5,
-                 random_state=None):
+    def __init__(
+        self,
+        outcome_learner=None,
+        effect_learner=None,
+        propensity_learner=ElasticNetPropensityModel(),
+        ate_alpha=0.05,
+        control_name=0,
+        n_fold=5,
+        random_state=None,
+    ):
         """Initialize an R-learner classifier.
 
         Args:
@@ -353,10 +410,13 @@ class BaseRClassifier(BaseRLearner):
             ate_alpha=ate_alpha,
             control_name=control_name,
             n_fold=n_fold,
-            random_state=random_state)
+            random_state=random_state,
+        )
 
         if (outcome_learner is None) and (effect_learner is None):
-            raise ValueError("Either the outcome learner or the effect learner must be specified.")
+            raise ValueError(
+                "Either the outcome learner or the effect learner must be specified."
+            )
 
     def fit(self, X, treatment, y, p=None, sample_weight=None, verbose=True):
         """Fit the treatment effect and outcome models of the R learner.
@@ -375,7 +435,9 @@ class BaseRClassifier(BaseRLearner):
         X, treatment, y = convert_pd_to_np(X, treatment, y)
         check_treatment_vector(treatment, self.control_name)
         if sample_weight is not None:
-            assert len(sample_weight) == len(y), "Data length must be equal for sample_weight and the input data"
+            assert len(sample_weight) == len(
+                y
+            ), "Data length must be equal for sample_weight and the input data"
             sample_weight = convert_pd_to_np(sample_weight)
         self.t_groups = np.unique(treatment[treatment != self.control_name])
         self.t_groups.sort()
@@ -392,8 +454,10 @@ class BaseRClassifier(BaseRLearner):
         self.vars_t = {}
 
         if verbose:
-            logger.info('generating out-of-fold CV outcome estimates')
-        yhat = cross_val_predict(self.model_mu, X, y, cv=self.cv, method='predict_proba', n_jobs=-1)[:, 1]
+            logger.info("generating out-of-fold CV outcome estimates")
+        yhat = cross_val_predict(
+            self.model_mu, X, y, cv=self.cv, method="predict_proba", n_jobs=-1
+        )[:, 1]
 
         for group in self.t_groups:
             mask = (treatment == group) | (treatment == self.control_name)
@@ -419,9 +483,14 @@ class BaseRClassifier(BaseRLearner):
                 self.vars_t[group] = diff_t.var()
 
             if verbose:
-                logger.info('training the treatment effect model for {} with R-loss'.format(group))
-            self.models_tau[group].fit(X_filt, (y_filt - yhat_filt) / (w - p_filt),
-                                       sample_weight=weight)
+                logger.info(
+                    "training the treatment effect model for {} with R-loss".format(
+                        group
+                    )
+                )
+            self.models_tau[group].fit(
+                X_filt, (y_filt - yhat_filt) / (w - p_filt), sample_weight=weight
+            )
 
     def predict(self, X, p=None):
         """Predict treatment effects.
@@ -442,15 +511,17 @@ class BaseRClassifier(BaseRLearner):
 
 
 class XGBRRegressor(BaseRRegressor):
-    def __init__(self,
-                 early_stopping=True,
-                 test_size=0.3,
-                 early_stopping_rounds=30,
-                 effect_learner_objective='rank:pairwise',
-                 effect_learner_n_estimators=500,
-                 random_state=42,
-                 *args,
-                 **kwargs):
+    def __init__(
+        self,
+        early_stopping=True,
+        test_size=0.3,
+        early_stopping_rounds=30,
+        effect_learner_objective="rank:pairwise",
+        effect_learner_n_estimators=500,
+        random_state=42,
+        *args,
+        **kwargs,
+    ):
         """Initialize an R-learner regressor with XGBoost model using pairwise ranking objective.
 
         Args:
@@ -464,7 +535,7 @@ class XGBRRegressor(BaseRRegressor):
             effect_learner_n_estimators (int, optional): number of trees to fit for the effect learner (default = 500)
         """
 
-        assert isinstance(random_state, int), 'random_state should be int.'
+        assert isinstance(random_state, int), "random_state should be int."
 
         objective, metric = get_xgboost_objective_metric(effect_learner_objective)
         self.effect_learner_objective = objective
@@ -477,11 +548,13 @@ class XGBRRegressor(BaseRRegressor):
 
         super().__init__(
             outcome_learner=XGBRegressor(random_state=random_state, *args, **kwargs),
-            effect_learner=XGBRegressor(objective=self.effect_learner_objective,
-                                        n_estimators=self.effect_learner_n_estimators,
-                                        random_state=random_state,
-                                        *args,
-                                        **kwargs)
+            effect_learner=XGBRegressor(
+                objective=self.effect_learner_objective,
+                n_estimators=self.effect_learner_n_estimators,
+                random_state=random_state,
+                *args,
+                **kwargs,
+            ),
         )
 
     def fit(self, X, treatment, y, p=None, sample_weight=None, verbose=True):
@@ -500,8 +573,14 @@ class XGBRRegressor(BaseRRegressor):
         X, treatment, y = convert_pd_to_np(X, treatment, y)
         check_treatment_vector(treatment, self.control_name)
         # initialize equal sample weight if it's not provided, for simplicity purpose
-        sample_weight = convert_pd_to_np(sample_weight) if sample_weight is not None else convert_pd_to_np(np.ones(len(y)))
-        assert len(sample_weight) == len(y), "Data length must be equal for sample_weight and the input data"
+        sample_weight = (
+            convert_pd_to_np(sample_weight)
+            if sample_weight is not None
+            else convert_pd_to_np(np.ones(len(y)))
+        )
+        assert len(sample_weight) == len(
+            y
+        ), "Data length must be equal for sample_weight and the input data"
         self.t_groups = np.unique(treatment[treatment != self.control_name])
         self.t_groups.sort()
 
@@ -517,7 +596,7 @@ class XGBRRegressor(BaseRRegressor):
         self.vars_t = {}
 
         if verbose:
-            logger.info('generating out-of-fold CV outcome estimates')
+            logger.info("generating out-of-fold CV outcome estimates")
         yhat = cross_val_predict(self.model_mu, X, y, cv=self.cv, n_jobs=-1)
 
         for group in self.t_groups:
@@ -532,31 +611,64 @@ class XGBRRegressor(BaseRRegressor):
             sample_weight_filt = sample_weight[treatment_mask]
 
             if verbose:
-                logger.info('training the treatment effect model for {} with R-loss'.format(group))
+                logger.info(
+                    "training the treatment effect model for {} with R-loss".format(
+                        group
+                    )
+                )
 
             if self.early_stopping:
-                X_train_filt, X_test_filt, y_train_filt, y_test_filt, yhat_train_filt, yhat_test_filt, \
-                    w_train, w_test, p_train_filt, p_test_filt, sample_weight_train_filt, sample_weight_test_filt \
-                    = train_test_split(
-                        X_filt, y_filt, yhat_filt, w, p_filt, sample_weight_filt,
-                        test_size=self.test_size, random_state=self.random_state
-                    )
+                (
+                    X_train_filt,
+                    X_test_filt,
+                    y_train_filt,
+                    y_test_filt,
+                    yhat_train_filt,
+                    yhat_test_filt,
+                    w_train,
+                    w_test,
+                    p_train_filt,
+                    p_test_filt,
+                    sample_weight_train_filt,
+                    sample_weight_test_filt,
+                ) = train_test_split(
+                    X_filt,
+                    y_filt,
+                    yhat_filt,
+                    w,
+                    p_filt,
+                    sample_weight_filt,
+                    test_size=self.test_size,
+                    random_state=self.random_state,
+                )
 
                 weight = sample_weight_filt
-                self.models_tau[group].fit(X=X_train_filt,
-                                           y=(y_train_filt - yhat_train_filt) / (w_train - p_train_filt),
-                                           sample_weight=sample_weight_train_filt * ((w_train - p_train_filt) ** 2),
-                                           eval_set=[(X_test_filt,
-                                                      (y_test_filt - yhat_test_filt) / (w_test - p_test_filt))],
-                                           sample_weight_eval_set=[sample_weight_test_filt * ((w_test - p_test_filt) ** 2)],
-                                           eval_metric=self.effect_learner_eval_metric,
-                                           early_stopping_rounds=self.early_stopping_rounds,
-                                           verbose=verbose)
+                self.models_tau[group].fit(
+                    X=X_train_filt,
+                    y=(y_train_filt - yhat_train_filt) / (w_train - p_train_filt),
+                    sample_weight=sample_weight_train_filt
+                    * ((w_train - p_train_filt) ** 2),
+                    eval_set=[
+                        (
+                            X_test_filt,
+                            (y_test_filt - yhat_test_filt) / (w_test - p_test_filt),
+                        )
+                    ],
+                    sample_weight_eval_set=[
+                        sample_weight_test_filt * ((w_test - p_test_filt) ** 2)
+                    ],
+                    eval_metric=self.effect_learner_eval_metric,
+                    early_stopping_rounds=self.early_stopping_rounds,
+                    verbose=verbose,
+                )
 
             else:
-                self.models_tau[group].fit(X_filt, (y_filt - yhat_filt) / (w - p_filt),
-                                           sample_weight=sample_weight_filt * ((w - p_filt) ** 2),
-                                           eval_metric=self.effect_learner_eval_metric)
+                self.models_tau[group].fit(
+                    X_filt,
+                    (y_filt - yhat_filt) / (w - p_filt),
+                    sample_weight=sample_weight_filt * ((w - p_filt) ** 2),
+                    eval_metric=self.effect_learner_eval_metric,
+                )
 
             diff_c = y_filt[w == 0] - yhat_filt[w == 0]
             diff_t = y_filt[w == 1] - yhat_filt[w == 1]
@@ -564,4 +676,3 @@ class XGBRRegressor(BaseRRegressor):
             sample_weight_filt_t = sample_weight_filt[w == 1]
             self.vars_c[group] = get_weighted_variance(diff_c, sample_weight_filt_c)
             self.vars_t[group] = get_weighted_variance(diff_t, sample_weight_filt_t)
-
